@@ -21,14 +21,22 @@ except ImportError:  # pragma: no cover
 
 class ThingyList(list):
     def distinct(self, key):
-        def __get_value(document):
-            if hasattr(document, "__dict__"):
-                document = document.__dict__
-            return document.get(key)
+        def __get_value(item):
+            if isinstance(item, BaseThingy):
+                item = item.view()
+            return item.get(key)
 
-        values = set(__get_value(result) for result in self)
+        values = set(__get_value(item) for item in self)
         return list(values)
 
+    def view(self, name="defaults"):
+        def __view(item):
+            if not isinstance(item, BaseThingy):
+                raise TypeError(f"Can't view type {type(item)}.")
+            return item.view(name)
+
+        return [__view(item) for item in self]
+
 
 class BaseThingy(DatabaseThingy):
     """Represents a document in a collection"""
@@ -153,6 +161,28 @@ class BaseThingy(DatabaseThingy):
         cursor = cls.find(filter, *args, **kwargs)
         return cursor.first()
 
+    @classmethod
+    def delete_many(cls, filter=None, *args, **kwargs):
+        return cls.collection.delete_many(filter, *args, **kwargs)
+
+    @classmethod
+    def delete_one(cls, filter=None, *args, **kwargs):
+        if filter is not None and not isinstance(filter, Mapping):
+            filter = {"_id": filter}
+
+        return cls.collection.delete_one(filter, *args, **kwargs)
+
+    @classmethod
+    def update_many(cls, filter, update, *args, **kwargs):
+        return cls.collection.update_many(filter, update, *args, **kwargs)
+
+    @classmethod
+    def update_one(cls, filter, update, *args, **kwargs):
+        if filter is not None and not isinstance(filter, Mapping):
+            filter = {"_id": filter}
+
+        return cls.collection.update_one(filter, update, *args, **kwargs)
+
     @property
     def id(self):
         return self.__dict__.get("id") or self._id
@@ -184,19 +214,41 @@ class Thingy(BaseThingy):
                 cls.collection.create_index(keys, **kwargs)
 
     @classmethod
-    def find_one_and_replace(cls, *args, **kwargs):
+    def find_one_and_replace(cls, filter, replacement, *args, **kwargs):
+        if filter is not None and not isinstance(filter, Mapping):
+            filter = {"_id": filter}
+
         kwargs.setdefault("return_document", ReturnDocument.AFTER)
-        result = cls.collection.find_one_and_replace(*args, **kwargs)
+        result = cls.collection.find_one_and_replace(
+            filter, replacement, *args, **kwargs
+        )
         if result is not None:
             return cls(result)
 
-    def save(self, force_insert=False):
+    @classmethod
+    def find_one_and_update(cls, filter, replacement, *args, **kwargs):
+        if filter is not None and not isinstance(filter, Mapping):
+            filter = {"_id": filter}
+
+        kwargs.setdefault("return_document", ReturnDocument.AFTER)
+        result = cls.collection.find_one_and_update(
+            filter, replacement, *args, **kwargs
+        )
+        if result is not None:
+            return cls(result)
+
+    def save(self, force_insert=False, refresh=False):
         data = self.__dict__
+        collection = self.get_collection()
+
         if self.id is not None and not force_insert:
             filter = {"_id": self.id}
-            self.get_collection().replace_one(filter, data, upsert=True)
+            collection.replace_one(filter, data, upsert=True)
         else:
-            self.get_collection().insert_one(data)
+            collection.insert_one(data)
+
+        if refresh:
+            self.__dict__ = collection.find_one(self.id)
         return self
 
 
@@ -216,19 +268,41 @@ class AsyncThingy(BaseThingy):
                 await cls.collection.create_index(keys, **kwargs)
 
     @classmethod
-    async def find_one_and_replace(cls, *args, **kwargs):
+    async def find_one_and_replace(cls, filter, replacement, *args, **kwargs):
+        if filter is not None and not isinstance(filter, Mapping):
+            filter = {"_id": filter}
+
         kwargs.setdefault("return_document", ReturnDocument.AFTER)
-        result = await cls.collection.find_one_and_replace(*args, **kwargs)
+        result = await cls.collection.find_one_and_replace(
+            filter, replacement, *args, **kwargs
+        )
         if result is not None:
             return cls(result)
 
-    async def save(self, force_insert=False):
+    @classmethod
+    async def find_one_and_update(cls, filter, replacement, *args, **kwargs):
+        if filter is not None and not isinstance(filter, Mapping):
+            filter = {"_id": filter}
+
+        kwargs.setdefault("return_document", ReturnDocument.AFTER)
+        result = await cls.collection.find_one_and_update(
+            filter, replacement, *args, **kwargs
+        )
+        if result is not None:
+            return cls(result)
+
+    async def save(self, force_insert=False, refresh=False):
         data = self.__dict__
+        collection = self.get_collection()
+
         if self.id is not None and not force_insert:
             filter = {"_id": self.id}
-            await self.get_collection().replace_one(filter, data, upsert=True)
+            await collection.replace_one(filter, data, upsert=True)
         else:
-            await self.get_collection().insert_one(data)
+            await collection.insert_one(data)
+
+        if refresh:
+            self.__dict__ = await collection.find_one(self.id)
         return self
 
 
@@ -245,12 +319,18 @@ def disconnect(*args, **kwargs):
 
 def create_indexes():
     """Create indexes registered on all :class:`Thingy`"""
+    tasks = []
+
     for cls in registry:
         if issubclass(cls, Thingy):
             cls.create_indexes()
         if issubclass(cls, AsyncThingy):
             coroutine = cls.create_indexes()
-            asyncio.create_task(coroutine)
+            task = asyncio.create_task(coroutine)
+            tasks.append(task)
+
+    if tasks:
+        return asyncio.wait(tasks)
 
 
 __all__ = ["AsyncThingy", "Thingy", "connect", "create_indexes"]
